@@ -417,11 +417,14 @@ extract_data_get_extraction_requested (ExtractData *extract_data,
 }
 
 
-static GFileInfo *
-_g_file_info_create_from_entry (struct archive_entry *entry,
-			        ExtractData          *extract_data)
+static void
+_g_file_set_attributes_from_entry (GFile                *file,
+				   struct archive_entry *entry,
+				   ExtractData          *extract_data,
+				   GCancellable         *cancellable)
 {
 	GFileInfo *info;
+	GError    *error = NULL;
 
 	info = g_file_info_new ();
 
@@ -475,66 +478,12 @@ _g_file_info_create_from_entry (struct archive_entry *entry,
 
 	g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE, archive_entry_mode (entry));
 
-	return info;
-}
-
-
-static gboolean
-_g_file_set_attributes_from_info (GFile         *file,
-				  GFileInfo     *info,
-				  GCancellable  *cancellable,
-				  GError       **error)
-{
-	return g_file_set_attributes_from_info (file, info, G_FILE_QUERY_INFO_NONE, cancellable, error);
-}
-
-
-static gboolean
-_g_file_set_attributes_from_entry (GFile                 *file,
-				   struct archive_entry  *entry,
-				   ExtractData           *extract_data,
-				   GCancellable          *cancellable,
-				   GError               **error)
-{
-	GFileInfo *info;
-	gboolean   result;
-
-	info = _g_file_info_create_from_entry (entry, extract_data);
-	result = _g_file_set_attributes_from_info (file, info, cancellable, error);
-
-	g_object_unref (info);
-
-	return result;
-}
-
-
-static gboolean
-restore_modification_time (GHashTable    *created_folders,
-			   GCancellable  *cancellable,
-			   GError       **error)
-{
-	GHashTableIter iter;
-	gpointer       key, value;
-	gboolean       result = TRUE;
-
-	g_hash_table_iter_init (&iter, created_folders);
-	while (result && g_hash_table_iter_next (&iter, &key, &value)) {
-		GFile     *file = key;
-		GFileInfo *original_info = value;
-		GFileInfo *info;
-
-		if (g_file_info_get_attribute_status (original_info, G_FILE_ATTRIBUTE_TIME_MODIFIED) != G_FILE_ATTRIBUTE_STATUS_SET)
-			continue;
-
-		info = g_file_info_new ();
-		g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, g_file_info_get_attribute_uint64 (original_info, G_FILE_ATTRIBUTE_TIME_MODIFIED));
-		g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, g_file_info_get_attribute_uint32 (original_info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC));
-		result = _g_file_set_attributes_from_info (file, info, cancellable, error);
-
-		g_object_unref (info);
+	if (! g_file_set_attributes_from_info (file, info, 0, cancellable, &error)) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
 	}
 
-	return result;
+	g_object_unref (info);
 }
 
 
@@ -546,7 +495,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 	ExtractData          *extract_data;
 	LoadData             *load_data;
 	GHashTable           *checked_folders;
-	GHashTable           *created_folders;
 	struct archive       *a;
 	struct archive_entry *entry;
 	int                   r;
@@ -555,7 +503,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 	load_data = LOAD_DATA (extract_data);
 
 	checked_folders = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
-	created_folders = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, g_object_unref);
 	fr_archive_progress_set_total_files (load_data->archive, extract_data->n_files_to_extract);
 
 	a = archive_read_new ();
@@ -565,7 +512,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 	while ((r = archive_read_next_header (a, &entry)) == ARCHIVE_OK) {
 		const char    *pathname;
 		char          *fullpath;
-		const char    *relative_path;
 		GFile         *file;
 		GFile         *parent;
 		GOutputStream *ostream;
@@ -585,12 +531,7 @@ extract_archive_thread (GSimpleAsyncResult *result,
 		}
 
 		fullpath = (*pathname == '/') ? g_strdup (pathname) : g_strconcat ("/", pathname, NULL);
-		relative_path = _g_path_get_relative_basename_safe (fullpath, extract_data->base_dir, extract_data->junk_paths);
-		if (relative_path == NULL) {
-			archive_read_data_skip (a);
-			continue;
-		}
-		file = g_file_get_child (extract_data->destination, relative_path);
+		file = g_file_get_child (extract_data->destination, _g_path_get_relative_basename (fullpath, extract_data->base_dir, extract_data->junk_paths));
 
 		/* honor the skip_older and overwrite options */
 
@@ -674,22 +615,14 @@ extract_archive_thread (GSimpleAsyncResult *result,
 
 			linkname = archive_entry_hardlink (entry);
 			if (linkname != NULL) {
-				char        *link_fullpath;
-				const char  *relative_path;
-				GFile       *link_file;
-				char        *oldname;
-				char        *newname;
-				int          r;
+				char  *link_fullpath;
+				GFile *link_file;
+				char  *oldname;
+				char  *newname;
+				int    r;
 
 				link_fullpath = (*linkname == '/') ? g_strdup (linkname) : g_strconcat ("/", linkname, NULL);
-				relative_path = _g_path_get_relative_basename_safe (link_fullpath, extract_data->base_dir, extract_data->junk_paths);
-				if (relative_path == NULL) {
-					g_free (link_fullpath);
-					archive_read_data_skip (a);
-					continue;
-				}
-
-				link_file = g_file_get_child (extract_data->destination, relative_path);
+				link_file = g_file_get_child (extract_data->destination, _g_path_get_relative_basename (link_fullpath, extract_data->base_dir, extract_data->junk_paths));
 				oldname = g_file_get_path (link_file);
 				newname = g_file_get_path (file);
 
@@ -736,15 +669,8 @@ extract_archive_thread (GSimpleAsyncResult *result,
 						load_data->error = g_error_copy (local_error);
 					g_error_free (local_error);
 				}
-				else {
-					GFileInfo *info;
-
-					info = _g_file_info_create_from_entry (entry, extract_data);
-					_g_file_set_attributes_from_info (file, info, cancellable, &load_data->error);
-					g_hash_table_insert (created_folders, g_object_ref (file), g_object_ref (info));
-
-					g_object_unref (info);
-				}
+				else
+					_g_file_set_attributes_from_entry (file, entry, extract_data, cancellable);
 				archive_read_data_skip (a);
 				break;
 
@@ -763,7 +689,7 @@ extract_archive_thread (GSimpleAsyncResult *result,
 				if (r != ARCHIVE_EOF)
 					load_data->error = g_error_new_literal (FR_ERROR, FR_ERROR_COMMAND_ERROR, archive_error_string (a));
 				else
-					_g_file_set_attributes_from_entry (file, entry, extract_data, cancellable, &load_data->error);
+					_g_file_set_attributes_from_entry (file, entry, extract_data, cancellable);
 				break;
 
 			case AE_IFLNK:
@@ -793,9 +719,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 		}
 	}
 
-	if (load_data->error == NULL)
-		restore_modification_time (created_folders, cancellable, &load_data->error);
-
 	if ((load_data->error == NULL) && (r != ARCHIVE_EOF))
 		load_data->error = g_error_new_literal (FR_ERROR, FR_ERROR_COMMAND_ERROR, archive_error_string (a));
 	if (load_data->error == NULL)
@@ -803,7 +726,6 @@ extract_archive_thread (GSimpleAsyncResult *result,
 	if (load_data->error != NULL)
 		g_simple_async_result_set_from_error (result, load_data->error);
 
-	g_hash_table_unref (created_folders);
 	g_hash_table_unref (checked_folders);
 	archive_read_free (a);
 	extract_data_free (extract_data);
