@@ -19,6 +19,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <config.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
@@ -27,6 +28,9 @@
 #include <glib/gprintf.h>
 #include <glib-object.h>
 #include "glib-utils.h"
+#if ENABLE_MAGIC
+#  include <magic.h>
+#endif
 
 
 #define MAX_PATTERNS 128
@@ -919,12 +923,6 @@ _g_path_remove_level (const gchar *path)
 	if (p < 0)
 		return NULL;
 
-	/* ignore the first slash if it's the last character,
-	 * this way /a/b/ is treated as /a/b */
-
-	if ((ptr[p] == '/') && (p > 0))
-		p--;
-
 	while ((p > 0) && (ptr[p] != '/'))
 		p--;
 	if ((p == 0) && (ptr[p] == '/'))
@@ -1022,46 +1020,6 @@ _g_path_get_relative_basename (const char *path,
 }
 
 
-#define ISDOT(c) ((c) == '.')
-#define ISSLASH(c) ((c) == '/')
-
-
-static const char *
-sanitize_filename (const char *file_name)
-{
-	size_t      prefix_len;
-	char const *p;
-
-	prefix_len = 0;
-	for (p = file_name; *p; ) {
-		if (ISDOT (p[0]) && ISDOT (p[1]) && (ISSLASH (p[2]) || !p[2]))
-			prefix_len = p + 2 - file_name;
-
-		do {
-			char c = *p++;
-			if (ISSLASH (c))
-				break;
-		}
-		while (*p);
-	}
-
-	p = file_name + prefix_len;
-	while (ISSLASH (*p))
-		p++;
-
-	return p;
-}
-
-
-const char *
-_g_path_get_relative_basename_safe (const char *path,
-				    const char *base_dir,
-				    gboolean    junk_paths)
-{
-	return sanitize_filename (_g_path_get_relative_basename (path, base_dir, junk_paths));
-}
-
-
 gboolean
 _g_filename_is_hidden (const gchar *name)
 {
@@ -1097,7 +1055,8 @@ _g_filename_get_extension (const char *filename)
 	ext = filename + p;
 	if (ext - 4 > filename) {
 		const char *test = ext - 4;
-		if (strncmp (test, ".tar", 4) == 0)
+		/* .tar.rz cannot be uncompressed in one step */
+		if ((strncmp (test, ".tar", 4) == 0) && (strncmp (ext, ".rz", 2) != 0))
 			ext = ext - 4;
 	}
 	return ext;
